use serenity::{
    async_trait,
    model::gateway::Ready,
    model::{
        application::command::{Command, CommandOptionType},
        prelude::ResumedEvent,
    },
    prelude::{Context, EventHandler},
};

use crate::database::Client as DatabaseClient;

const MAX_ARGS_NUMBER: u32 = 25;

pub struct Handler<'a> {
    pub database: &'a DatabaseClient,
}

#[async_trait]
impl EventHandler for Handler<'_> {
    async fn ready(&self, ctx: Context, ready: Ready) {
        println!("Successfully connected as {}", ready.user.name);

        match Command::create_global_application_command(&ctx.http, |command| {
            command
                .name("create_tag")
                .description("Add a new tag")
                .create_option(|option| {
                    option
                        .name("tag")
                        .description("The tag to create")
                        .kind(CommandOptionType::String)
                        .required(true)
                })
        })
        .await
        {
            Ok(_) => println!("Successfully registered create_tag command"),
            Err(e) => println!("Failed to register create_tag command : {e}"),
        };

        match Command::create_global_application_command(&ctx.http, |command| {
            let command = command
                .name("bulk_create_tag")
                .description("Add multiples tags");

            for i in 0..MAX_ARGS_NUMBER {
                command.create_option(|option| {
                    option
                        .name(format!("tag{}", i + 1))
                        .description("A new tag to add")
                        .kind(CommandOptionType::String)
                        .required(i == 0)
                });
            }

            command
        })
        .await
        {
            Ok(_) => println!("Successfully registered bulk_create_tag command"),
            Err(e) => println!("Failed to register bulk_create_tag command: {e}"),
        };

        match Command::create_global_application_command(&ctx.http, |command| {
            command
                .name("delete_tag")
                .description("Delete a tag")
                .create_option(|option| {
                    option
                        .name("tag")
                        .description("The tag to delete")
                        .kind(CommandOptionType::String)
                        .required(true)
                })
        })
        .await
        {
            Ok(_) => println!("Successfully registered delete_tag command"),
            Err(e) => println!("Failed to register delete_tag command: {e}"),
        };
    }

    async fn resume(&self, _: Context, _: ResumedEvent) {
        println!("Successfully reconnected.")
    }
}
